//---------------------------------------------------------------------------
//
// PROJECT : chromawaves
//
//
// AUTOR   : Martin Steen
//           email: martin@martin-steen.de
//
//
//----------------------------------------------------------------------------

#define APP_NAME "chromawaves"

#ifdef _WIN32
#include <windows.h>
#endif

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <stdlib.h>

using namespace std;

#include <GLinclude.h>
#include <math.h>
#include <CVector3T.h>
#include <COpenGL.h>
#include "CGLApplication.h"
#include "CHeightField.h"

static float sYrot;
static float sZrot;
static float sXrot;




// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : CGLApplication
//
//
//
// ---------------------------------------------------------------------------

CGLApplication::CGLApplication()
{
	mInitFlag = false;
	mAutoScale = false;
	mAutoRotate = true;
	mShowInterface = true;
	mFullscreen = false;
	mAutoBlubs = false;
 	mFieldRes = 128;
	mPolygonMode = GL_FILL;
	srand(time(NULL));
}

// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : FileExists
//
//
//
// ---------------------------------------------------------------------------

static bool FileExists(const char* f)
{
	ifstream k(f, ios::binary);

	if (k.good())
	{
		k.close();
		return true;
	}
	cout << "***** File not found: " << f << endl;
	return false;
}



// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       :
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::SetResolution(int w, int h)
{
	mWindowWidth  = w;
	mWindowHeight = h;

	if (!mInitFlag)
	{
		mInitFlag = true;
	}
}

// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       :
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::InitApplication(int argc, char* argv[])
{
	bool bChangeRot = false;

  gOpenGL       = new COpenGL;

	mFilemap    = NULL;
	mScale      = 0.1;

	mXrotFlag = false;
	mYrotFlag = false;
	mZrotFlag = false;

	unsigned int GlMode = EGL_DEPTH | EGL_DOUBLE | EGL_RGBA;
	
	for (int i = 1; i < argc; i++)
	{
		string argvi = argv[i];	
	
		if (argvi == "-auto")
		{
			mAutoBlubs = true;
		}
		else
		if (argvi == "-fullscreen")
		{
			GlMode |= EGL_FULLSCREEN;
		}
		else
		if (argvi == "-q1")
		{
			mFieldRes = 64;
		}	
		else
		if (argvi == "-q2")
		{
			mFieldRes = 96;
		}	
		else
		if (argvi == "-q3")
		{
			mFieldRes = 128;
		}	
		else
		if (argvi == "-q4")
		{
			mFieldRes = 192;
		}	
		else
		if (argvi == "-q5")
		{
			mFieldRes = 250;
		}	
		else
		if (argvi == "-db")
		{
			GlMode &= ~EGL_DOUBLE;
		}	
		else
		if (argvi == "-dp")
		{
			GlMode &= ~EGL_DEPTH;
		}	
		else
		if (argvi == "-rgba")
		{
			GlMode &= ~EGL_RGBA;
		}	
		else	
		if ((argvi == "-help") ||
        (argvi == "-?"))

		{
			cout << "chromawaves <options>" << endl;
			cout << " options:" << endl;
			cout << "   -fullscreen = run fullscreen mode" << endl;
			cout << "   -auto = make autowaves" << endl;
			cout << "   -q1   = lowest quality" << endl;
			cout << "   -q2   = lower quality" << endl;
			cout << "   -q3   = medium quality ( default)" << endl;
			cout << "   -q4   = better quality" << endl;
			cout << "   -q5   = best quality" << endl;
			cout << "   -db   = no double buffering" << endl;
			cout << "   -dp   = no depth buffer" << endl;
			cout << "version 1.0 / written by Martin Steen" << endl;
			exit(0);			
		}
	}
	



	if (!bChangeRot)
	{
		mXrotFlag = true;
		mYrotFlag = true;
		mZrotFlag = true;
		mXrotVal  = 0.02;
		mYrotVal  = 0.04;
		mZrotVal  = 0.2;
	}

	gOpenGL->mCamera.mStandort.z = -10.0;
	gOpenGL->InitWindowGLUT(argc, argv, APP_NAME, 800, 600, GlMode);

	sXrot = 0;
	sYrot = 0;
  
	gOpenGL->mDrawmode = EDRAWMODE_ENVMAP;
	gOpenGL->InitGLUTCallback(10);
}


//---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : 
//
//
//
//---------------------------------------------------------------------------

static float Random01()
{
	return rand() / float(RAND_MAX);
}

//---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : 
//
//
//
//---------------------------------------------------------------------------

static float Random(float Low, float High)
{
	 return (Random01() * (High - Low)) + Low;
}

//---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : 
//
//
//
//---------------------------------------------------------------------------

void CGLApplication::NewBlub()
{
	CBlub blub0;
	static int StaticId;

	blub0.mPos.x   = Random01();
  blub0.mPos.y   = Random01();
	blub0.mPhase   = 0;
	blub0.mImpuls  = Random(15.0, 20.0);
	blub0.mFreq    = Random(1.0, 2.0);
	blub0.mStage   = 0.1;
	blub0.mId      = StaticId++;
	blub0.mQ1       = Random(1.0, 2.0);
	blub0.mQ2       = Random(1.0, 2.0);
		

	mBlubVec.push_back(blub0);

}

//---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : 
//
//
//
//---------------------------------------------------------------------------

void CGLApplication::RunBlubs()
{
	vector<CBlub> RemoveVec;
	static int Max;
	static int sAutoBlubTime;

	if (mBlubVec.size() > Max)
	{
		Max = mBlubVec.size();
	}

	for (list<CBlub>::iterator ib = mBlubVec.begin(); ib != mBlubVec.end(); ++ib)
	{
		ib->mPhase -= 0.1;
		ib->mImpuls *= 0.97;
		ib->mStage  += 0.2;

		if (ib->mImpuls < 0.1)
		{
			RemoveVec.push_back(*ib);
		}
	}

	for (vector<CBlub>::iterator ib = RemoveVec.begin(); ib != RemoveVec.end(); ++ib)
	{
		mBlubVec.remove(*ib);
	}

	if (mBlubVec.size() < Max)
	{
		NewBlub();
	}

	if (mAutoBlubs)
	{
		if (mBlubVec.size() < 8)
		{
			if (sAutoBlubTime == 0)
			{
				NewBlub();
				sAutoBlubTime = Random(50, 200);	
			}
			else
			{
				sAutoBlubTime--;
			}
		}	
	}

		
}


// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       :
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::Draw3DObjects()
{
	if (!mInitFlag)
	{
		return;
	}
	glPushMatrix();



	//glRotatef(mXrotAuto,      1.0, 0.0, 0.0); // X-Rotation
	//mXrotAuto += 0.13;
	
	//glRotatef(mZrotAuto,      0.0, 0.0, 1.0); // Z-Rotation
	//mZrotAuto += 0.17;
	
	//glRotatef(mYrotAuto,      0.0, 1.0, 0.0); // Y-Rotation
	//mYrotAuto += 0.1;


	glDisable(GL_LIGHTING);

	glColor4f(1.0, 0.0, 0.0, 1.0);

  GLfloat WhiteMaterial[] = { 0.7, 0.8, 0.7, 1.0 };
  glMaterialfv(GL_FRONT, GL_DIFFUSE, WhiteMaterial);


	glRotatef(sXrot, 1.0, 0.0, 0.0);
	glRotatef(sZrot, 0.0, 0.0, 1.0);
	glRotatef(sYrot, 0.0, 1.0, 0.0);
	glScalef(mScale, mScale, mScale);
	
	sXrot += 0.1;
	sYrot += 0.09;

  glPushMatrix();

	//vector<CVector2<float> > Blubs;
	//CVector2<float> Blub0;

	DrawHeightFieldColor(mFieldRes, mFieldRes, mBlubVec);
	RunBlubs();

	//cout << "W=" << mGreyField.mWidth << " H=" << mGreyField.mHeight << endl;

	//DrawHeightField(&mGreyField);


  glPopMatrix();

}

// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : MouseMotionLeft
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::MouseMotionLeft(int dx, int dy)
{
	sXrot += (float) dy / 3;
	sYrot -= (float) dx / 3;

	//cout << "Xrot= " << sXrot << " Yrot=" << sYrot << endl;
}


// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : MouseMotionRight
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::MouseMotionRight(int dx, int dy)
{
	Zoom(1.0 + (float) dy / 100);
}


// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : MouseWheel
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::MouseWheel(int d)
{
  if (d == 0)
  {
		Zoom(1.05);
  }
  else
  {
		Zoom(1.0 / 1.05);
  }
}

// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : Zoom
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::Zoom(float f)
{
	mScale *= f;
	if (mScale > 10)
	{
		mScale = 10;
	}
  if (mScale < 0.1)
	{
		mScale = 0.1;
	}
}

// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : LeftMouseButtonDown
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::LeftMouseButtonDown()
{
	return;

}

// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : RightMouseButtonDown
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::RightMouseButtonDown()
{
}

// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : KeyboardAction
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::KeyboardAction(unsigned char key)
{
	switch (key)
	{
		case ' ':

			NewBlub();
			break;

		case 'p':

			if (mPolygonMode == GL_LINE)
			{
				mPolygonMode = GL_FILL;
			}
			else
			{
				mPolygonMode = GL_LINE;
			}
			glPolygonMode(GL_FRONT_AND_BACK, mPolygonMode);
			break;


		case 'q':
		case 27:

			exit(0);
			break;
	}
}


// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : Draw2DObjects
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::Draw2DObjects()
{
	if (mInitFlag)
	{

	}
}

// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : ManageInterface
//
//
//
// ---------------------------------------------------------------------------

void CGLApplication::ManageInterface(CGL_Mouse* Mouse)
{
	/*
	if (mInitFlag)
	{
		if (mShowInterface)
		{
			gG3Dinterface->SendCommand(EG3DcomRollover, Mouse);
			gG3Dinterface->SendCommand(EG3DcomDraw);
		}
	}
	*/
}

