//---------------------------------------------------------------------------
//
// PROJECT : chromawaves
//
//
// AUTOR   : Martin Steen
//           email: martin@martin-steen.de
//
//
//----------------------------------------------------------------------------

#include <iostream>
#include <fstream>
#include <string>
#include <vector>

using namespace std;

#include <GLinclude.h>
#include <math.h>
#include <CVector3T.h>
#include <COpenGL.h>
#include "CGLApplication.h"
#include "CHeightField.h"

// ---------------------------------------------------------------------------
//
// KLASSE        : CGLApplication
// METHODE       : DrawHeightFieldColor
//
//
//
// ---------------------------------------------------------------------------


void CGLApplication::DrawHeightFieldColor(int PsegS, int PsegT, list<CBlub>& Blub)
{
	PsegS += 1;
	PsegT += 1;

	float vs = 0.0;
	int nElems = PsegS * PsegT;
	unsigned short* IndexArray = new unsigned short[PsegT * 2];
	CVector3<float>*  VertexArray = new CVector3<float>[nElems];
	CVector3<float>*  ColorArray  = new CVector3<float>[nElems];


	CVector3<float>*  va = VertexArray;
	CVector3<float>*  ca = ColorArray;

	float StepS = 1.0 / PsegS;
	float StepT = 1.0 / PsegT;


	for (int s = 0; s < PsegS; s++)
	{
		float vt = 0.0;

		for (int t = 0; t < PsegT; t++)
		{
			CVector3<float> Vertex;
		
			va->x = (vs - 0.5) * 100;
			va->y = (vt - 0.5) * 100;
      va->z = 0;

			float PhasSum1 = 0;
			float PhasSum2 = 0;

			for (list<CBlub>::iterator ib = mBlubVec.begin(); ib != mBlubVec.end(); ++ib)
			{
				float as = vs - ib->mPos.x;
				float at = vt - ib->mPos.y;

				float ds = as * 10;
				float dt = at * 10;
				float SqrDist = (ds * ds) + (dt * dt);

				float dist = sqrt(SqrDist);

				if (dist < ib->mStage)
				{
					va->z    +=  sin(dist*ib->mFreq + ib->mPhase) * ib->mImpuls / (SqrDist+1);
					PhasSum1 +=  sin(dist*(ib->mFreq*ib->mQ2) + ib->mPhase) * ib->mImpuls / (SqrDist+1);
					PhasSum2 +=  sin(dist*(ib->mFreq*ib->mQ1) + ib->mPhase) * ib->mImpuls / (SqrDist+1);
				}				
			}

			ca->x = fabs(va->z) /    10;
			ca->y = fabs(PhasSum2) / 10;
      ca->z = fabs(PhasSum1) / 10;
			ca++;

			va++;

			vt += StepT;
		}
		vs += StepS;


	}

  glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_VERTEX_ARRAY);

	glColorPointer(3, GL_FLOAT, 0, ColorArray);
	glVertexPointer(3, GL_FLOAT, 0, VertexArray);

	int k = 0;
	for (int s = 0; s < PsegS-1; s++)
	{
		unsigned short* ix = IndexArray;
		for (int t = 0; t < PsegT; t++)
		{
			*(ix++) = k;
			*(ix++) = k + PsegT;

			k++;
		}
		glDrawElements(GL_TRIANGLE_STRIP, PsegT * 2, GL_UNSIGNED_SHORT, IndexArray);
	}

	delete[] IndexArray;
	delete[] VertexArray;
	delete[] ColorArray;

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_COLOR_ARRAY);

}

